/*******************************************************************************
 * Copyright (c) 2009 Mia-Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Nicolas Payneau (Mia-Software) - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.modisco.workflow.ui;

import org.eclipse.core.runtime.IPath;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationListener;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTabGroup;
import org.eclipse.debug.ui.ILaunchConfigurationDialog;
import org.eclipse.debug.ui.ILaunchConfigurationTab;
import org.eclipse.gmt.modisco.workflow.Activator;
import org.eclipse.gmt.modisco.workflow.Constants;
import org.eclipse.gmt.modisco.workflow.ui.common.Common;

/**
 * 
 * @author Nicolas Payneau
 */
public class MoDiscoWorkflowTabGroup extends
		AbstractLaunchConfigurationTabGroup {
	private ILaunchConfigurationTab wfTab;
	private ILaunchConfigurationTab export;
	private ILaunchConfigurationTab common;

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTabGroup#createTabs(org.eclipse.debug.ui.ILaunchConfigurationDialog, java.lang.String)
	 */
	public void createTabs(ILaunchConfigurationDialog dialog, String mode) {
		this.wfTab = new WorkflowTransformationTab();
		this.export = new MoDiscoExportTab();
		this.common = new MoDiscoWorkflowCommonTab();
		setTabs(new ILaunchConfigurationTab[] {this.wfTab,this.export,this.common});
	}
	
	public void initializeFrom(ILaunchConfiguration configuration) {
		this.common.initializeFrom(configuration);
		((WorkflowTransformationTab) this.wfTab).setResourcePath(((MoDiscoWorkflowCommonTab)this.common).getResourcePath());
		this.wfTab.initializeFrom(configuration);
		DebugPlugin.getDefault().getLaunchManager().addLaunchConfigurationListener(new ILaunchConfigurationListener(){
			public void launchConfigurationAdded(ILaunchConfiguration configuration) {}
			public void launchConfigurationChanged(ILaunchConfiguration configuration) {}
			public void launchConfigurationRemoved(ILaunchConfiguration configuration) {
				String resourcePath = null;
				if (!configuration.isLocal()) {				
					IPath path = configuration.getFile().getLocation().removeLastSegments(1).append(configuration.getName()).addFileExtension(Constants.workflowExtension);
					resourcePath = path.toString();
				} else {
					resourcePath = Activator.getDefault().getStateLocation().append(configuration.getName()).addFileExtension(Constants.workflowExtension).toString();
				}
				((WorkflowTransformationTab) MoDiscoWorkflowTabGroup.this.wfTab).getController().removeResource(resourcePath);
			}
		});
		this.export.initializeFrom(configuration);
	}
		
	@Override
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		((WorkflowTransformationTab) this.wfTab).getController().changeWorkflowEngine(configuration);
		super.performApply(configuration);
		((WorkflowTransformationTab) this.wfTab).getController().saveWorkflowResource(configuration,false);
		((WorkflowTransformationTab) this.wfTab).getController().moveWorkflowResource(((MoDiscoWorkflowCommonTab)this.common).getResourcePath());
		//For rename configuration
		if(!configuration.getOriginal().getName().equals(configuration.getName())){
			String resourcePath = null;
			if (!configuration.isLocal()) {				
				IPath path = configuration.getFile().getLocation().removeLastSegments(1).append(configuration.getName()).addFileExtension(Constants.workflowExtension);
				resourcePath = path.toString();
			} else {
				resourcePath = Activator.getDefault().getStateLocation().append(configuration.getName()).addFileExtension(Constants.workflowExtension).toString();
			}
			((WorkflowTransformationTab) this.wfTab).getController().moveWorkflowResource(resourcePath);
		}
		Common.getInstance().refreshPackageExplorer();		
	}
}
