/*******************************************************************************
 * Copyright (c) 2009 Mia-Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Nicolas Payneau (Mia-Software) - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.modisco.workflow.ui.common;

import org.eclipse.gmt.modisco.workflow.Activator;
import org.eclipse.gmt.modisco.workflow.Constants;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

/**
 * This class defines remove dialog box
 * @author Nicolas Payneau
 */
public class RemoveDialog extends Dialog{
	private Constants mode = Constants.Cancel;
	private String transformationName = null;

	/**
	 * Class constructor
	 */
	public RemoveDialog(Shell shell, String transformationName){
		super(shell);
		Window.setDefaultImage(Activator.getImageDescriptor(
				Constants.pathLittleImage).createImage());
		this.transformationName = transformationName;
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		GridData gridDataLabel = new GridData();
		gridDataLabel.verticalSpan = 10;
		gridDataLabel.horizontalAlignment = SWT.CENTER;
		gridDataLabel.verticalAlignment = SWT.CENTER;
		
		GridLayout dialogLayout = new GridLayout();
		dialogLayout.numColumns = 1;
		dialogLayout.horizontalSpacing = 9;
		parent.setLayout(dialogLayout);
		
		Label labelInformation = new Label(parent, SWT.NONE);
		labelInformation.setText("Are you sure you want to delete transformation "+this.transformationName+" ?");
		labelInformation.setLayoutData(gridDataLabel);
		
		GridData dataButtons = new GridData();
		dataButtons.horizontalAlignment = SWT.CENTER;
		Composite compositeButton = new Composite(parent,SWT.NULL);
		compositeButton.setLayoutData(dataButtons);
		GridLayout layoutButton = new GridLayout();
		layoutButton.numColumns = 3;
		layoutButton.horizontalSpacing = 9;
		compositeButton.setLayout(layoutButton);
		Button deleteFromLauncher = new Button(compositeButton, SWT.NONE);
		deleteFromLauncher.setText("Delete from launchers");
		Button deleteFromWorkflow = new Button(compositeButton, SWT.NONE);
		deleteFromWorkflow.setText("Delete from workflow");
		Button cancel = new Button(compositeButton, SWT.NONE);
		cancel.setText("Cancel");
		cancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				RemoveDialog.this.close();
			}
		});
		deleteFromWorkflow.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				RemoveDialog.this.mode = Constants.deleteFromWorkflowMode;
				RemoveDialog.this.close();
			}
		});
		deleteFromLauncher.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				RemoveDialog.this.mode = Constants.deleteFromLauncherMode;
				RemoveDialog.this.close();
			}
		});
		this.getShell().setText("Remove MoDiscoWork");
		return labelInformation;
	}
	
	/**
	 * Allows to retrieve the delete mode (deleteFromWorkflow or deleteFromLauncher)
	 * @return Return the delete mode
	 */
	public Constants getMode() {
		return this.mode;
	}
	
	
	
}
