/*******************************************************************************
 * Copyright (c) 2009 Mia-Software.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Nicolas Payneau (Mia-Software) - initial API and implementation
 *******************************************************************************/
package org.eclipse.gmt.modisco.workflow.ui.common;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * This class defines export dialog box
 * It allows to capture project name, symbolic name or base package name
 * @author Nicolas Payneau
 */
@SuppressWarnings("restriction")
public class ExportDialog extends Dialog{

	private Text baseNameText;
	private Text projectNameText;
	private Text symbolicNameText;
	private boolean export;
	private Composite compositeButton = null;
	private Button exportButton = null;
	private String baseName;
	private String projectName;
	private String symbolicName;
	private String defaultName;
	
	public ExportDialog(String defaultName, Shell parentShell) {
		super(parentShell);
		this.defaultName = defaultName;
	}

	@Override
	protected Control createContents(Composite parent) {
		parent.getShell().setSize(400,150);
		parent.getShell().setLocation(Common.getInstance().centerShell(parent.getShell().getDisplay(),parent.getShell()));  
		GridLayout layoutParent = new GridLayout();
		parent.setLayout(layoutParent);
		layoutParent.numColumns = 3;
		layoutParent.makeColumnsEqualWidth = false;
		GridData gdText = new GridData();
		gdText.horizontalAlignment = GridData.FILL;
		gdText.horizontalSpan = 2;
		gdText.grabExcessHorizontalSpace = true;
		gdText.verticalAlignment = GridData.CENTER;
		Label labelProjectName = new Label(parent, SWT.NONE);
		labelProjectName.setText("Project name");
		this.projectNameText = new Text(parent, SWT.BORDER);
		this.projectNameText.setLayoutData(gdText);
		this.projectNameText.setText(this.defaultName.toLowerCase());
		this.projectNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ExportDialog.this.updateButton();				
			}
		});
		Label labelSymbolicName = new Label(parent, SWT.NONE);
		labelSymbolicName.setText("Symbolic name");
		this.symbolicNameText = new Text(parent, SWT.BORDER);
		this.symbolicNameText.setLayoutData(gdText);
		this.symbolicNameText.setText(this.defaultName.toLowerCase());
		this.symbolicNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ExportDialog.this.updateButton();				
			}
		});
		Label labelBase = new Label(parent, SWT.NONE);
		labelBase.setText("Base package name");
		this.baseNameText = new Text(parent, SWT.BORDER);
		this.baseNameText.setLayoutData(gdText);
		this.baseNameText.setText("org.eclipse."+this.defaultName.toLowerCase());
		this.baseNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				ExportDialog.this.updateButton();				
			}
		});
		createCompositeButton(parent);
		this.getShell().setText("Export to discoverer plugin");
		return parent;
	}
	
	/**
	 * Create composite button
	 * @param parent Composite parent
	 */
	private void createCompositeButton(Composite parent) {
		GridData gdButtonExport = new GridData();
		gdButtonExport.grabExcessHorizontalSpace = true;
		gdButtonExport.verticalAlignment = GridData.CENTER;
		gdButtonExport.horizontalAlignment = GridData.END;
		GridData gridData5 = new GridData();
		gridData5.horizontalAlignment = GridData.BEGINNING;
		gridData5.grabExcessHorizontalSpace = true;
		gridData5.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData gridData4 = new GridData();
		gridData4.horizontalSpan = 3;
		gridData4.verticalAlignment = GridData.FILL;
		gridData4.grabExcessVerticalSpace = true;
		gridData4.horizontalAlignment = GridData.FILL;
		this.compositeButton = new Composite(parent, SWT.NONE);
		this.compositeButton.setLayoutData(gridData4);
		this.compositeButton.setLayout(gridLayout);
		this.exportButton = new Button(this.compositeButton, SWT.NONE);
		this.exportButton.setText("Export");
		this.exportButton.setLayoutData(gdButtonExport);
		this.exportButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				super.widgetSelected(e);
				IStatus status = verifyAllName();
				if(status.isOK()){
					ExportDialog.this.export = true;
					ExportDialog.this.projectName = ExportDialog.this.projectNameText.getText().trim();
					ExportDialog.this.baseName = ExportDialog.this.baseNameText.getText().trim();
					ExportDialog.this.symbolicName = ExportDialog.this.symbolicNameText.getText().trim();
					ExportDialog.this.close();
				}else{
					MessageDialog.openError(getShell(),"Name is not valid", status.getMessage());
				}
			}
		});
		Button buttonCancel = new Button(this.compositeButton, SWT.NONE);
		buttonCancel.setText("Cancel");
		buttonCancel.setLayoutData(gridData5);
		buttonCancel.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				super.widgetSelected(e);
				ExportDialog.this.export = false;
				ExportDialog.this.close();
			}
		});
		updateButton();
	}

	/**
	 * Update button
	 */
	private void updateButton() {
		if(this.projectNameText.getText().trim().length()>0&&this.baseNameText.getText().trim().length()>0){
			this.exportButton.setEnabled(true);
		}else{
			this.exportButton.setEnabled(false);
		}
	}
	
	/**
	 * Allows to verify all the fields Text are valid
	 * @return Status of verification
	 */
	private IStatus verifyAllName() {
		IStatus status = null;
		status = JavaConventions.validatePackageName(this.symbolicNameText.getText(),CompilerOptions.VERSION_1_6, CompilerOptions.VERSION_1_6);
		if(status.isOK()){
			status = JavaConventions.validatePackageName(this.baseNameText.getText(),CompilerOptions.VERSION_1_6, CompilerOptions.VERSION_1_6);
		}
		return status;
	}

	//Getters
	public boolean isExport() {
		return export;
	}
	
	public String getProjectName(){
		return this.projectName;
	}
	
	public String getBaseName() {
		return this.baseName;
	}

	public String getSymbolicName() {
		return this.symbolicName;
	}
	
}
